@@ -5,22 +5,25 @@ import os
 import sys
 from collections import defaultdict
 from contextlib import contextmanager
+from itertools import chain
 from pathlib import Path
 from threading import RLock
-from typing import Any, Dict, Generator, Iterator, NoReturn, Optional, Sequence, cast
+from typing import TYPE_CHECKING, Any, Dict, Generator, Iterator, Literal, NoReturn, Optional, Sequence, cast
 
 from cachetools import cached
 from packaging.requirements import Requirement
-from pyproject_api import BackendFailed, CmdStatus, Frontend
+from pyproject_api import (
+    BackendFailed,
+    CmdStatus,
+    Frontend,
+    MetadataForBuildEditableResult,
+    MetadataForBuildWheelResult,
+)
 
-from tox.config.sets import EnvConfigSet
-from tox.execute.api import ExecuteStatus
 from tox.execute.pep517_backend import LocalSubProcessPep517Executor
 from tox.execute.request import StdinSource
 from tox.plugin import impl
-from tox.tox_env.api import ToxEnvCreateArgs
 from tox.tox_env.errors import Fail
-from tox.tox_env.package import Package, PackageToxEnv
 from tox.tox_env.python.package import (
     EditableLegacyPackage,
     EditablePackage,
@@ -28,17 +31,20 @@ from tox.tox_env.python.package import (
     SdistPackage,
     WheelPackage,
 )
-from tox.tox_env.register import ToxEnvRegister
-from tox.tox_env.runner import RunToxEnv
+from tox.tox_env.python.virtual_env.api import VirtualEnv
 from tox.util.file_view import create_session_view
 
-from ..api import VirtualEnv
 from .util import dependencies_with_extras, dependencies_with_extras_from_markers
 
-if sys.version_info >= (3, 8):  # pragma: no cover (py38+)
-    from importlib.metadata import Distribution, PathDistribution
-else:  # pragma: no cover (<py38)
-    from importlib_metadata import Distribution, PathDistribution
+if TYPE_CHECKING:
+    from tox.config.sets import EnvConfigSet
+    from tox.execute.api import ExecuteStatus
+    from tox.tox_env.api import ToxEnvCreateArgs
+    from tox.tox_env.package import Package, PackageToxEnv
+    from tox.tox_env.register import ToxEnvRegister
+    from tox.tox_env.runner import RunToxEnv
+
+from importlib.metadata import Distribution, PathDistribution
 
 if sys.version_info >= (3, 11):  # pragma: no cover (py311+)
     import tomllib
@@ -64,8 +70,8 @@ class ToxBackendFailed(Fail, BackendFailed):
         )
 
 
-class BuildEditableNotSupported(RuntimeError):
-    """raised when build editable is not supported"""
+class BuildEditableNotSupportedError(RuntimeError):
+    """raised when build editable is not supported."""
 
 
 class ToxCmdStatus(CmdStatus):
@@ -89,12 +95,13 @@ class ToxCmdStatus(CmdStatus):
 
 
 class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
-    """local file system python virtual environment via the virtualenv package"""
+    """local file system python virtual environment via the virtualenv package."""
 
     def __init__(self, create_args: ToxEnvCreateArgs) -> None:
         super().__init__(create_args)
         self._frontend_: Pep517VirtualEnvFrontend | None = None
         self.builds: defaultdict[str, list[EnvConfigSet]] = defaultdict(list)
+        self.call_require_hooks: set[str] = set()
         self._distribution_meta: PathDistribution | None = None
         self._package_dependencies: list[Requirement] | None = None
         self._package_name: str | None = None
@@ -103,7 +110,7 @@ class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
         self._package_paths: set[Path] = set()
 
     @staticmethod
-    def id() -> str:
+    def id() -> str:  # noqa: A003
         return "virtualenv-pep-517"
 
     @property
@@ -117,15 +124,32 @@ class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
         self.conf.add_config(
             keys=["meta_dir"],
             of_type=Path,
-            default=lambda conf, name: self.env_dir / ".meta",  # noqa: U100
+            default=lambda conf, name: self.env_dir / ".meta",  # noqa: ARG005
             desc="directory where to put the project metadata files",
         )
         self.conf.add_config(
             keys=["pkg_dir"],
             of_type=Path,
-            default=lambda conf, name: self.env_dir / "dist",  # noqa: U100
+            default=lambda conf, name: self.env_dir / "dist",  # noqa: ARG005
             desc="directory where to put project packages",
         )
+        for key in ("sdist", "wheel", "editable"):
+            self._add_config_settings(key)
+
+    def _add_config_settings(self, build_type: str) -> None:
+        # config settings passed to PEP-517-compliant build backend https://peps.python.org/pep-0517/#config-settings
+        keys = {
+            "sdist": ["get_requires_for_build_sdist", "build_sdist"],
+            "wheel": ["get_requires_for_build_wheel", "prepare_metadata_for_build_wheel", "build_wheel"],
+            "editable": ["get_requires_for_build_editable", "prepare_metadata_for_build_editable", "build_editable"],
+        }
+        for key in keys.get(build_type, []):
+            self.conf.add_config(
+                keys=[f"config_settings_{key}"],
+                of_type=Dict[str, str],
+                default=None,  # type: ignore[arg-type]
+                desc=f"config settings passed to the {key} backend API endpoint",
+            )
 
     @property
     def pkg_dir(self) -> Path:
@@ -148,28 +172,31 @@ class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
     def register_run_env(self, run_env: RunToxEnv) -> Generator[tuple[str, str], PackageToxEnv, None]:
         yield from super().register_run_env(run_env)
         build_type = run_env.conf["package"]
+        self.call_require_hooks.add(build_type)
         self.builds[build_type].append(run_env.conf)
 
     def _setup_env(self) -> None:
         super()._setup_env()
-        if "editable" in self.builds:
+        if "sdist" in self.call_require_hooks or "external" in self.call_require_hooks:
+            self._setup_build_requires("sdist")
+        if "wheel" in self.call_require_hooks:
+            self._setup_build_requires("wheel")
+        if "editable" in self.call_require_hooks:
             if not self._frontend.optional_hooks["build_editable"]:
-                raise BuildEditableNotSupported
-            build_requires = self._frontend.get_requires_for_build_editable().requires
-            self._install(build_requires, PythonPackageToxEnv.__name__, "requires_for_build_editable")
-        if "wheel" in self.builds:
-            build_requires = self._frontend.get_requires_for_build_wheel().requires
-            self._install(build_requires, PythonPackageToxEnv.__name__, "requires_for_build_wheel")
-        if "sdist" in self.builds or "external" in self.builds:
-            build_requires = self._frontend.get_requires_for_build_sdist().requires
-            self._install(build_requires, PythonPackageToxEnv.__name__, "requires_for_build_sdist")
+                raise BuildEditableNotSupportedError
+            self._setup_build_requires("editable")
+
+    def _setup_build_requires(self, of_type: str) -> None:
+        settings: ConfigSettings = self.conf[f"config_settings_get_requires_for_build_{of_type}"]
+        requires = getattr(self._frontend, f"get_requires_for_build_{of_type}")(config_settings=settings).requires
+        self._install(requires, PythonPackageToxEnv.__name__, f"requires_for_build_{of_type}")
 
     def _teardown(self) -> None:
         executor = self._frontend.backend_executor
         if executor is not None:  # pragma: no branch
             try:
                 if executor.is_alive:
-                    self._frontend._send("_exit")  # try first on amicable shutdown
+                    self._frontend._send("_exit")  # try first on amicable shutdown  # noqa: SLF001
             except SystemExit:  # pragma: no cover  # if already has been interrupted ignore
                 pass
             finally:
@@ -181,29 +208,37 @@ class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
         super()._teardown()
 
     def perform_packaging(self, for_env: EnvConfigSet) -> list[Package]:
-        """build the package to install"""
+        """Build the package to install."""
         try:
             deps = self._load_deps(for_env)
-        except BuildEditableNotSupported:
+        except BuildEditableNotSupportedError:
+            self.call_require_hooks.remove("editable")
             targets = [e for e in self.builds.pop("editable") if e["package"] == "editable"]
             names = ", ".join(sorted({t.env_name for t in targets if t.env_name}))
-            logging.error(
-                f"package config for {names} is editable, however the build backend {self._frontend.backend}"
-                f" does not support PEP-660, falling back to editable-legacy - change your configuration to it",
+
+            logging.error(  # noqa: TRY400
+                "package config for %s is editable, however the build backend %s does not support PEP-660, falling "
+                "back to editable-legacy - change your configuration to it",
+                names,
+                cast(Pep517VirtualEnvFrontend, self._frontend_).backend,
             )
             for env in targets:
-                env._defined["package"].value = "editable-legacy"  # type: ignore
+                env._defined["package"].value = "editable-legacy"  # type: ignore[attr-defined]  # noqa: SLF001
                 self.builds["editable-legacy"].append(env)
+            self._run_state["setup"] = False  # force setup again as we need to provision wheel to get dependencies
             deps = self._load_deps(for_env)
         of_type: str = for_env["package"]
         if of_type == "editable-legacy":
             self.setup()
-            deps = [*self.requires(), *self._frontend.get_requires_for_build_sdist().requires] + deps
+            config_settings: ConfigSettings = self.conf["config_settings_get_requires_for_build_sdist"]
+            sdist_requires = self._frontend.get_requires_for_build_sdist(config_settings=config_settings).requires
+            deps = [*self.requires(), *sdist_requires, *deps]
             package: Package = EditableLegacyPackage(self.core["tox_root"], deps)  # the folder itself is the package
         elif of_type == "sdist":
             self.setup()
             with self._pkg_lock:
-                sdist = self._frontend.build_sdist(sdist_directory=self.pkg_dir).sdist
+                config_settings = self.conf["config_settings_build_sdist"]
+                sdist = self._frontend.build_sdist(sdist_directory=self.pkg_dir, config_settings=config_settings).sdist
                 sdist = create_session_view(sdist, self._package_temp_path)
                 self._package_paths.add(sdist)
                 package = SdistPackage(sdist, deps)
@@ -215,17 +250,19 @@ class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
             else:
                 self.setup()
                 method = "build_editable" if of_type == "editable" else "build_wheel"
+                config_settings = self.conf[f"config_settings_{method}"]
                 with self._pkg_lock:
                     wheel = getattr(self._frontend, method)(
                         wheel_directory=self.pkg_dir,
                         metadata_directory=self.meta_folder_if_populated,
-                        config_settings=self._wheel_config_settings,
+                        config_settings=config_settings,
                     ).wheel
                     wheel = create_session_view(wheel, self._package_temp_path)
                     self._package_paths.add(wheel)
                 package = (EditablePackage if of_type == "editable" else WheelPackage)(wheel, deps)
         else:  # pragma: no cover # for when we introduce new packaging types and don't implement
-            raise TypeError(f"cannot handle package type {of_type}")  # pragma: no cover
+            msg = f"cannot handle package type {of_type}"
+            raise TypeError(msg)  # pragma: no cover
         return [package]
 
     @property
@@ -283,8 +320,7 @@ class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
             reqs = self.get_package_dependencies(for_env)
             name = self.get_package_name(for_env)
         extras: set[str] = for_env["extras"]
-        deps = dependencies_with_extras(reqs, extras, name)
-        return deps
+        return dependencies_with_extras(reqs, extras, name)
 
     def get_package_dependencies(self, for_env: EnvConfigSet) -> list[Requirement]:
         with self._pkg_lock:
@@ -304,17 +340,21 @@ class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
     def _ensure_meta_present(self, for_env: EnvConfigSet) -> None:
         if self._distribution_meta is not None:  # pragma: no branch
             return  # pragma: no cover
-        self.setup()
-        end = self._frontend
-        if for_env["package"] == "editable":
-            dist_info = end.prepare_metadata_for_build_editable(self.meta_folder, self._wheel_config_settings).metadata
-        else:
-            dist_info = end.prepare_metadata_for_build_wheel(self.meta_folder, self._wheel_config_settings).metadata
-        self._distribution_meta = Distribution.at(str(dist_info))
+        # even if we don't build a wheel we need the requirements for it should we want to build its metadata
+        target: Literal["editable", "wheel"] = "editable" if for_env["package"] == "editable" else "wheel"
+        self.call_require_hooks.add(target)
 
-    @property
-    def _wheel_config_settings(self) -> ConfigSettings | None:
-        return {"--build-option": []}
+        self.setup()
+        hook = getattr(self._frontend, f"prepare_metadata_for_build_{target}")
+        config: ConfigSettings = self.conf[f"config_settings_prepare_metadata_for_build_{target}"]
+        result: MetadataForBuildWheelResult | MetadataForBuildEditableResult | None = hook(self.meta_folder, config)
+        if result is None:
+            config = self.conf[f"config_settings_build_{target}"]
+            dist_info_path, _, __ = self._frontend.metadata_from_built(self.meta_folder, target, config)
+            dist_info = str(dist_info_path)
+        else:
+            dist_info = str(result.metadata)
+        self._distribution_meta = Distribution.at(dist_info)
 
     def requires(self) -> tuple[Requirement, ...]:
         return self._frontend.requires
@@ -326,33 +366,41 @@ class Pep517VirtualEnvFrontend(Frontend):
         self._tox_env = env
         self._backend_executor_: LocalSubProcessPep517Executor | None = None
         into: dict[str, Any] = {}
-        pkg_cache = cached(
-            into,
-            key=lambda *args, **kwargs: "wheel" if "wheel_directory" in kwargs else "sdist",  # noqa: U100
-        )
-        self.build_wheel = pkg_cache(self.build_wheel)  # type: ignore
-        self.build_sdist = pkg_cache(self.build_sdist)  # type: ignore
-        self.build_editable = pkg_cache(self.build_editable)  # type: ignore
+
+        for hook in chain(
+            (f"get_requires_for_build_{build_type}" for build_type in ["editable", "wheel", "sdist"]),
+            (f"prepare_metadata_for_build_{build_type}" for build_type in ["editable", "wheel"]),
+            (f"build_{build_type}" for build_type in ["editable", "wheel", "sdist"]),
+        ):  # wrap build methods in a cache wrapper
+
+            def key(*args: Any, bound_return: str = hook, **kwargs: Any) -> str:  # noqa: ARG001
+                return bound_return
+
+            setattr(self, hook, cached(into, key=key)(getattr(self, hook)))
 
     @property
     def backend_cmd(self) -> Sequence[str]:
-        return ["python"] + self.backend_args
+        return ["python", *self.backend_args]
 
     def _send(self, cmd: str, **kwargs: Any) -> tuple[Any, str, str]:
         try:
-            if cmd in ("prepare_metadata_for_build_wheel", "prepare_metadata_for_build_editable"):
-                # given we'll build a wheel we might skip the prepare step
-                if "wheel" in self._tox_env.builds or "editable" in self._tox_env.builds:
-                    return None, "", ""  # will need to build wheel either way, avoid prepare
+            if self._can_skip_prepare(cmd):
+                return None, "", ""  # will need to build wheel either way, avoid prepare
             return super()._send(cmd, **kwargs)
         except BackendFailed as exception:
             raise exception if isinstance(exception, ToxBackendFailed) else ToxBackendFailed(exception) from exception
 
+    def _can_skip_prepare(self, cmd: str) -> bool:
+        # given we'll build a wheel we might skip the prepare step
+        return cmd in ("prepare_metadata_for_build_wheel", "prepare_metadata_for_build_editable") and (
+            "wheel" in self._tox_env.builds or "editable" in self._tox_env.builds
+        )
+
     @contextmanager
     def _send_msg(
         self,
         cmd: str,
-        result_file: Path,  # noqa: U100
+        result_file: Path,  # noqa: ARG002
         msg: str,
     ) -> Iterator[ToxCmdStatus]:
         with self._tox_env.execute_async(
@@ -369,7 +417,14 @@ class Pep517VirtualEnvFrontend(Frontend):
         if outcome is not None:  # pragma: no branch
             outcome.assert_success()
 
-    def _unexpected_response(self, cmd: str, got: Any, expected_type: Any, out: str, err: str) -> NoReturn:
+    def _unexpected_response(  # noqa: PLR0913
+        self,
+        cmd: str,
+        got: Any,
+        expected_type: Any,
+        out: str,
+        err: str,
+    ) -> NoReturn:
         try:
             super()._unexpected_response(cmd, got, expected_type, out, err)
         except BackendFailed as exception:
